@@ -11,7 +11,7 @@
 #' package versions (see the second example).
 #' @param install Whether to install missing packages from `repos`.
 #' @param load Whether to load packages (which is exactly *not* the point
-#' of this function, but hey, YMMV).
+#' of this package, but hey, YMMV).
 #' @param repos  The repository to use if installing packages; default
 #' is the RStudio repository.
 #'
@@ -21,15 +21,15 @@
 #' @examples \donttest{
 #' rock::checkPkgs('base');
 #'
-#' ### Require a version
-#' rock::checkPkgs(rock = "0.5.0");
+#' ### Require a specific version
+#' rock::checkPkgs(rock = "0.9.1");
 #'
 #' ### This will show the error message
 #' tryCatch(
 #'   rock::checkPkgs(
 #'     base = "99",
 #'     stats = "42.5",
-#'     ufs = 20
+#'     rock = 2000
 #'   ),
 #'   error = print
 #' );
@@ -38,6 +38,7 @@ checkPkgs <- function(...,
                       install = FALSE,
                       load = FALSE,
                       repos = "https://cran.rstudio.com") {
+
   vrsn <- unlist(list(...));
   if (is.null(names(vrsn))) {
     x <- vrsn;
@@ -46,15 +47,26 @@ checkPkgs <- function(...,
     x <- names(vrsn);
   }
 
-  res <- c();
+  presences <-
+    unlist(
+      lapply(
+        x,
+        requireNamespace,
+        quietly = TRUE
+      )
+    );
+  names(presences) <- x;
 
-  for (i in seq_along(x)) {
-    if (length(find.package(x[i]) > 0)) {
-      if (utils::compareVersion(as.character(utils::packageVersion(x[i])), vrsn[i]) < 0) {
-        res[x[i]] <- TRUE;
+  res <- rep(FALSE, length(x));
+  names(res) <- x;
+
+  for (i in x) {
+    if (presences[i]) {
+      if (utils::compareVersion(as.character(utils::packageVersion(i)), vrsn[i]) < 0) {
+        res[i] <- TRUE;
       }
     } else {
-      res[x[i]] <- TRUE;
+      res[i] <- TRUE;
     }
   }
   if (any(res)) {
@@ -64,13 +76,17 @@ checkPkgs <- function(...,
     } else {
       stop("Of package(s) ", vecTxtQ(x[res]),
            ", you need at least versions ", vecTxt(vrsn[res]),
-           ", respectively. Install those with:\n\n",
-           "install.packages(c(", vecTxtQ(x[res]), "));\n");
+           ", respectively. Install those with:\n\n  ",
+           "install.packages(c(",
+           vecTxtQ(x[res], lastDelimiter = ", "),
+           "));\n");
     }
   }
+
   if (load) {
     suppressMessages(invisible(lapply(x[!res],
                                       require)));
   }
+
   return(invisible(res));
 }
