@@ -7,11 +7,13 @@ parse_sources <- function(path,
                           removeSectionBreakRows = rock::opts$get('removeSectionBreakRows'),
                           removeIdentifierRows = rock::opts$get('removeIdentifierRows'),
                           removeEmptyRows = rock::opts$get('removeEmptyRows'),
+                          suppressDuplicateInstanceWarnings = rock::opts$get('suppressDuplicateInstanceWarnings'),
+                          filesWithYAML = NULL,
                           ignoreOddDelimiters = FALSE,
-                          checkClassInstanceIds = rock::opts$get(checkClassInstanceIds),
+                          checkClassInstanceIds = rock::opts$get("checkClassInstanceIds"),
                           mergeInductiveTrees = FALSE,
-                          encoding=rock::opts$get(encoding),
-                          silent=rock::opts$get(silent)) {
+                          encoding=rock::opts$get("encoding"),
+                          silent=rock::opts$get("silent")) {
 
   codeRegexes <- rock::opts$get(codeRegexes);
   idRegexes <- rock::opts$get(idRegexes);
@@ -74,7 +76,9 @@ parse_sources <- function(path,
            postponeDeductiveTreeBuilding = TRUE,
            removeSectionBreakRows = removeSectionBreakRows,
            removeIdentifierRows = removeIdentifierRows,
+           filesWithYAML = filesWithYAML,
            removeEmptyRows = removeEmptyRows,
+           mergeAttributes = FALSE,
            silent=silent);
 
   if (!silent) {
@@ -117,6 +121,24 @@ parse_sources <- function(path,
   #            })
   #   );
 
+  ### 2024-05-29: Get all the class identifiers from all the sources
+  res$convenience$allClassIds <-
+    unique(
+      unlist(
+        lapply(
+          res$parsedSources,
+          function(x) {
+            return(x$convenience$allClasses);
+          }
+        )
+      )
+    );
+
+  ### 2024-05-29: Stored just before refactoring to allow attributes to exist for multiple
+  ### classes
+  ### Also 2024-05-29: enabled again as we discovered that merge_utterances_and_attributes()
+  ### seems to be written for multiple classes with attributes already --- provided we can
+  ### deal with different column names for each attribute specification
   res$convenience$attributes <-
     rbind_df_list(
       lapply(
@@ -127,9 +149,203 @@ parse_sources <- function(path,
       )
     );
 
-    # dplyr::bind_rows(purrr::map(res$parsedSources,
-    #                             'attributesDf'));
+  ### 2024-05-29: Parse the attributes in all separate sources, organizing them per
+  ### class identifier, and then collapsing them over sources into one attribute dataframe
+  ### per class identifier.
 
+  res$convenience$attributesPerClass <-
+    lapply(
+      res$convenience$allClassIds,
+      function(currentClassId) {
+
+        listOfAttDfsForThisClassForAllSources <-
+          lapply(
+            res$parsedSources,
+            function(currentSource) {
+
+              if ((!all(is.na(currentSource$attributes))) &&
+                  (length(currentSource$attributes) > 0)) {
+
+                listOfDataframes <-
+                  lapply(
+                    currentSource$attributes,
+                    function(currentAttributeSpec) {
+                      if (currentClassId %in% names(currentAttributeSpec)) {
+                        return(as.data.frame(currentAttributeSpec, stringsAsFactors = FALSE));
+                      } else {
+                        return(NULL);
+                      }
+                    }
+                  );
+
+                if (length(unlist(listOfDataframes)) > 0) {
+
+                  attributeDfForThisClassInThisSource <-
+                    rbind_df_list(listOfDataframes);
+
+                  # attributeDfForThisClassInThisSource <-
+                  #   tryCatch(
+                  #     do.call(rbind,
+                  #             listOfDataframes),
+                  #     error = function(e) {
+                  #
+                  #       colCounts <-
+                  #         table(
+                  #           unlist(
+                  #             lapply(
+                  #               listOfDataframes,
+                  #               colnames
+                  #             )
+                  #           )
+                  #         );
+                  #
+                  #       stop("I could not parse the attributes into a data frame. At present, ",
+                  #            "I require that all attributes are specified for all class ",
+                  #            "instances - you may have omitted one (or more). Sorry! ",
+                  #            "The following columns appear the following number of ",
+                  #            "times: ", vecTxt(paste0(names(colCounts), " (", colCounts, " times)")),
+                  #            ".");
+                  #     });
+
+                } else {
+
+                  ### No attributes for this source
+                  return(NULL);
+
+                }
+
+              } else {
+
+                ### No attributes for this source
+                return(NULL);
+
+              }
+
+              return(attributeDfForThisClassInThisSource);
+
+            }
+          );
+
+        if (length(unlist(listOfAttDfsForThisClassForAllSources)) == 0) {
+          attributeDfsRbindedOverSources <- NULL;
+        } else {
+
+          attributeDfsRbindedOverSources <-
+            tryCatch(
+              rbind_df_list(listOfAttDfsForThisClassForAllSources),
+              # do.call(rbind,
+              #         listOfAttDfsForThisClassForAllSources),
+              error = function(e) {
+
+                colCounts <-
+                  table(
+                    unlist(
+                      lapply(
+                        listOfAttDfsForThisClassForAllSources,
+                        colnames
+                      )
+                    )
+                  );
+
+                stop("I could not parse the attributes into a data frame. At present, ",
+                     "I require that all attributes are specified for all class ",
+                     "instances - you may have omitted one (or more). Sorry! ",
+                     "The following columns appear the following number of ",
+                     "times: ", vecTxt(paste0(names(colCounts), " (", colCounts, " times)")),
+                     ".");
+              });
+        }
+
+        if (length(unlist(listOfAttDfsForThisClassForAllSources)) > 0) {
+
+          if (length(duplicated(attributeDfsRbindedOverSources[[currentClassId]])) > 0) {
+
+            res <-
+              do.call(
+                rbind,
+                lapply(
+                  sort(unique(attributeDfsRbindedOverSources[[currentClassId]])),
+                  function(instanceId) {
+
+                    instanceDf <-
+                      attributeDfsRbindedOverSources[
+                        attributeDfsRbindedOverSources[[currentClassId]] == instanceId,
+                      ];
+
+                    instanceList <-
+                      lapply(
+                        names(instanceDf),
+                        function(colName) {
+
+                          col <- instanceDf[, colName];
+
+                          nonMissingValues <-
+                            unique(col[!is.na(col)]);
+
+                          if (all(is.na(col))) {
+                            return(NA);
+                          } else if (all(!is.na(col)) && (length(nonMissingValues) == 1)) {
+                            return(nonMissingValues);
+                          } else if (length(nonMissingValues) == 1) {
+                            return(nonMissingValues);
+                          } else {
+                            ### Take the first non-NA element
+                            firstElement <- nonMissingValues[1];
+                            if (!suppressDuplicateInstanceWarnings) {
+                              warning("For instance '", instanceId, "' of class '",
+                                      currentClassId, "', attribute '",
+                                      colName, "' has different values: ",
+                                      vecTxtQ(nonMissingValues),
+                                      ". Taking the first element: '",
+                                      firstElement, "'.");
+                            }
+                            return(firstElement);
+                          }
+                        }
+                    );
+
+                    names(instanceList) <- names(instanceDf);
+
+                    return(
+                      as.data.frame(
+                        instanceList
+                      )
+                    );
+
+                  }
+                )
+              );
+
+          } else {
+            res <- attributeDfsRbindedOverSources;
+          }
+
+        } else {
+          res <- NA;
+        }
+
+        return(res);
+
+      }
+    );
+
+  ### Class instance identifiers are sometimes used without attributes; in that
+  ### case, this will be a data frame of 0 cols and 0 rows.
+  if (ncol(res$convenience$attributes) == length(res$convenience$allClassIds)) {
+    names(res$convenience$attributes) <- res$convenience$allClassIds;
+  }
+
+  ### 2024-05-29: What we just produced is actually the 'new attributeDf' except that it's
+  ### a list of Dfs organized per class identifier
+
+  # dplyr::bind_rows(purrr::map(res$parsedSources,
+  #                             'attributesDf'));
+
+  ### 2024-05-29: This no longer makes sense; keeping the code for now for future reference
+  res$convenience$attributesVars <- NA;
+
+  ### 2024-05-29: Given what we just discovered about merge_utterances_and_attributes()
+  ### this is 'reactivated' again
   res$convenience$attributesVars <-
     sort(unique(c(unlist(lapply(
       res$parsedSource,
@@ -296,6 +512,20 @@ parse_sources <- function(path,
     #                             'sourceDf'));
 
   ### Merge merged source dataframes
+
+  ### 2024-05-29: Replacing this with a new merging activity where we merge
+  ### the attributes in from the new res$convenience$attributes object using
+  ### the res$convenience$allClassIds
+#
+#   res$qdt <-
+#     res$sourceDf;
+#
+#   for (currentClassId in res$convenience$allClassIds) {
+#     if (currentClassId %in% names(res$qdt)) {
+#
+#     }
+#   }
+
   res$qdt <-
     rbind_df_list(
       lapply(
@@ -327,7 +557,7 @@ parse_sources <- function(path,
     #                  .id="originalSource");
 
   res$qdt[, res$convenience$codingLeaves] <-
-    lapply(res$qdt[, res$convenience$codingLeaves],
+    lapply(res$qdt[, res$convenience$codingLeaves, drop=FALSE],
            function(x) {
              return(ifelse(is.na(x),
                            0,
@@ -442,7 +672,7 @@ parse_sources <- function(path,
     c(specifiedClasses, unspecifiedClasses);
 
   ### Merge attributes with source dataframe
-  if (length(res$attributes) > 0) {
+  if (nrow(attributesDf) > 0) {
 
     qdtNew <-
       merge_utterances_and_attributes(
@@ -450,6 +680,7 @@ parse_sources <- function(path,
         classes = allClasses,
         attributesDf = res$attributesDf,
         checkClassInstanceIds = checkClassInstanceIds,
+        suppressDuplicateInstanceWarnings = suppressDuplicateInstanceWarnings,
         silent = silent
       );
 
