@@ -5,6 +5,9 @@
 #'
 #' @param x The string (e.g. a source)
 #' @param length The maximum length
+#' @param collapseResult Whether to collapse the result from a vector (with
+#' line breaks separating the elements) to a single character value (where
+#' the vector elements are glued together using `splitString`) or not.
 #' @param splitString The character to use to split lines.
 #'
 #' @return A character vector.
@@ -23,11 +26,14 @@
 #' );
 split_long_lines <- function(x,
                              length = 60,
+                             collapseResult = FALSE,
                              splitString = rock::opts$get("utteranceMarker")) {
 
   res <- strwrap(x, width=length);
 
-  res <- paste0(res, collapse=splitString);
+  if (collapseResult) {
+    res <- paste0(res, collapse=splitString);
+  }
 
   return(res);
 
